"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const meow = require("meow");
const ora = require("ora");
const path_1 = require("path");
const repl = require("repl");
const Mod_1 = require("./Mod");
const utils_1 = require("./utils");
const cli = meow(`
Usage
  $ npm-try [package ..]

Options
  --registry, -r Specify the registry that will be used when npm install.
  --out-dir, -o Create a self-contained project with the packages installed in the directory.
  --verbose, -v Verbose mode. Print debugging messages about their progress.

Examples
  $ npm-try lodash
  $ npm-try lodash@3
  $ npm-try lodash underscore
  $ npm-try lodash -o my-lodash
  `, {
    flags: {
        registry: {
            type: "string",
            alias: "r",
        },
        outDir: {
            type: "string",
            alias: "o",
        },
        verbose: {
            default: false,
            type: "boolean",
            alias: "v",
        },
    },
});
async function run() {
    const mods = cli.input.map((target) => new Mod_1.Mod(target));
    const path = await utils_1.createProjectDirectory();
    const defines = [];
    for (const mod of mods) {
        const spinner = ora(`Installing ${mod.target}...`).start();
        try {
            await utils_1.installModule(path, [mod.target], cli.flags);
            const succeed = `const ${mod.variableName} = require('${mod.packageName}')`;
            if (cli.flags.outDir) {
                spinner.succeed();
            }
            else {
                spinner.succeed(succeed);
            }
            defines.push(succeed);
        }
        catch (err) {
            spinner.fail(`Install ${mod.target} failed. Does package "${mod.packageName}" exist?`);
        }
    }
    if (cli.flags.outDir) {
        createProject(path, cli.flags.outDir, defines);
    }
    else {
        startREPL(path, mods);
    }
}
function createProject(path, to, defines) {
    const spinner = ora(`Creating project at ${to}...`).start();
    to = path_1.resolve(process.cwd(), to);
    const content = `'use strict'\n\n${defines.join("\n")}\n\n// Your awesome code here\n`;
    try {
        fs_extra_1.moveSync(path, to);
        fs_extra_1.writeFileSync(path_1.join(to, "index.js"), content);
        spinner.succeed("The project created at " + to);
    }
    catch (err) {
        spinner.fail(err.message);
    }
}
function startREPL(path, mods) {
    const r = repl.start({ prompt: "> " });
    initializeContext();
    function initializeContext() {
        r.context.module.paths.unshift(path_1.join(path, "node_modules"));
        mods.forEach((mod) => {
            try {
                const required = r.context.module.require(mod.packageName);
                if (mod.variableName) {
                    r.context[mod.variableName] = required;
                }
            }
            catch (err) {
            }
        });
    }
}
run();
